#!/bin/bash
#
# health-report.sh
#  Simple health reporting tool, supporting top and df reports ( v1 )
#
# Changelog
#  2014-03-26 - Nathan Isburgh <nathan@edgecloud.com>
#             * Version 2
#             * Support for mem report, collect mode and report mode
#  2014-03-25 - Nathan Isburgh <nathan@edgecloud.com>
#             * Initial version
#             * Support for top and df -h reports
#

# User configurable variables
DEBUG=1

#
# END OF CONFIGURATION VARIABLES
# NO MORE EDITS BELOW THIS LINE
#

# Functions
fail() {
  cleanup
  ECODE=${1:-1}
  EMSG=$2
  if [ -n "$EMSG" ]
  then
    printf "%s\n" $EMSG > /dev/stderr
  fi
  exit $ECODE
}

succeed() {
  cleanup
  exit 0
}

cleanup() {
  # Close log files, shutdown connections, etc..
  rm -f $TMPFILE
  return
}

debug() {
  if [ $DEBUG -eq 0 ] 
  then
    return
  fi
  printf " >>> %s %s\n" "$(date)" "$1" > /dev/stderr
}

usage() {
  cat << eof
Usage:
  $0 [-tmcd] [-r date] email
   -t         Send top report to the given email address
   -m         Send memory report to the given email address
   -d         Send df -h report to the given email address
   -c         Collect mode
   -r date    Report on date ( YYYY-MM-DD format )
   email      Email address for recipient of reports
eof
}

top-report() {
  echo -e "\nTop report\n------------------------" >> $TMPFILE
  top -bn 1 >> $TMPFILE
}

df-report() {
  echo -e "\ndf report\n------------------------" >> $TMPFILE
  df -h >> $TMPFILE
}

mem-report() {
  echo -e "\nMemory report\n------------------------" >> $TMPFILE
  ps -Ao comm -o %mem | tail -n +2 | sort -rnk 2 >> $TMPFILE
}

# Main body

# Add hook to call cleanup on exit automagically
trap cleanup TERM QUIT ABRT INT EXIT

if [ $# -lt 1 ]
then
  echo "Please supply email address" > /dev/stderr
  usage
  fail
fi

RUNTOP=0
RUNDF=0
RUNMEM=0
LOGFILE=""
COLLECT=0
REPORT=0

while getopts ":cr:tmd" opt; do
  case $opt in
    c)
      COLLECT=1
      LOGFILE="/tmp/health-report.$(date +%F)"
      ;;
    r)
      # XXX Do simple regex check of OPTARG to verify YYYY-MM-DD format
      REPORT=1
      LOGFILE="/tmp/health-report.$OPTARG"
      ;;
    t)
      RUNTOP=1
      ;;
    d)
      RUNDF=1
      ;;
    m)
      RUNMEM=1
      ;;
    \?)
      echo "Invalid option: -$OPTARG" > /dev/stderr
      usage
      fail 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument." > /dev/stderr
      usage
      fail 1
      ;;
  esac
done

eval EMAIL=\${$OPTIND}
TMPFILE=$(mktemp)

# Runs the reports..
if [ $RUNTOP -eq 1 ]
then
  top-report
fi
if [ $RUNDF -eq 1 ]
then
  df-report
fi
if [ $RUNMEM -eq 1 ]
then
  mem-report
fi

# If in collect mode, we always append current report date to the
# permanent log file..
if [ $COLLECT -eq 1 ]
then
  echo -e "\nHealth report - $(date)" >> $LOGFILE
  echo "===================================" >> $LOGFILE
  cat $TMPFILE >> $LOGFILE
fi

# If we're in report mode, then send an email of the log file requested
if [ $REPORT -eq 1 ]
then
  if [ -e $LOGFILE ]
  then
    mailx -s "Health report" $EMAIL < $LOGFILE
  else
    fail 2 "Requested report date does not exist: $LOGFILE"
  fi
elif [ $COLLECT -eq 0 ]
then
  # Not report mode, not collect mode
  mailx -s "Health report" $EMAIL < $TMPFILE
fi

# X Collect only mode ( TMPFILE >> LOGFILE )
# X Report only mode ( LOGFILE is emailed )
# X Collect and Report mode ( TMPFILE >> LOGFILE && email LOGFILE )
# X One-time ( no collect, no report - send email of TMPFILE )

succeed
