#!/bin/bash
#
# proc-count.sh
#  Simple tool to report [filtered] process counts
#
# Changelog
#  2014-03-25 - Nathan Isburgh <nathan@edgecloud.com>
#             * Initial version
#             * Support for basic counts and filters
#             * Support for CSV or TSV formats
#

# User configurable variables
DEBUG=1

RED="\033[31m"
NORMAL="\033[39m"


#
# END OF CONFIGURATION VARIABLES
# NO MORE EDITS BELOW THIS LINE
#

# Functions
fail() {
  cleanup
  ECODE=${1:-1}
  EMSG=$2
  if [ -n "$EMSG" ]
  then
    printf "%s\n" $EMSG > /dev/stderr
  fi
  exit $ECODE
}

succeed() {
  cleanup
  exit 0
}

cleanup() {
  # Close log files, shutdown connections, etc..
  rm -f $TMPFILE
  return
}

debug() {
  if [ $DEBUG -eq 0 ] 
  then
    return
  fi
  printf " >>> %s %s\n" "$(date)" "$1" > /dev/stderr
}

usage() {
  cat << eof
Usage:
  $0 [-f filter]... [-c] email
   -f filter   Filter processes by given value
   -c          Generate a CSV report, intead of TSV
   email       Email address for recipient of reports
eof
}

proc-report() {
  # Working with each filter, grab list of matching processes,
  # count them, and add to final report..
  if [ ${#FILTERS[*]} -eq 0 ]
  then
    if [ $OUTPUT = 'tsv' ]
    then
      ps -Ao comm | sort | uniq -c | awk '{print $2"\t"$1}' >> $TMPFILE
    else
      ps -Ao comm | sort | uniq -c | awk '{print $2","$1}' >> $TMPFILE
    fi
  fi
  for filter in ${FILTERS[*]}
  do
    echo -e "\n$filter\n===========================" >> $TMPFILE
    if [ $OUTPUT = 'tsv' ]
    then
      ps -Ao comm | fgrep $filter | sort | uniq -c | awk '{print $2"\t"$1}' >> $TMPFILE
    else
      ps -Ao comm | fgrep $filter | sort | uniq -c | awk '{print $2","$1}' >> $TMPFILE
    fi
  done
  mailx -s "Process Report" $EMAIL < $TMPFILE
}

# Main body

if [ $# -lt 1 ]
then
  echo "Please supply email address" > /dev/stderr
  usage
  fail
fi

# Array of filters, supplied by user at command line
FILTERS=()

# Report output format, tsv or csv
OUTPUT=tsv

while getopts ":f:c" opt; do
  case $opt in
    f)
      FILTERS+=("$OPTARG")
      ;;
    c)
      OUTPUT=csv
      ;;
    \?)
      echo -e "${RED}Invalid option: -$OPTARG${NORMAL}" > /dev/stderr
      usage
      fail 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument." > /dev/stderr
      usage
      fail 1
      ;;
  esac
done

eval EMAIL=\${$OPTIND}

TMPFILE=$(mktemp)
proc-report

succeed
